searchNodes=[{"doc":"Allows you to use arithmetical operations on any type that implements the proper protocols. For each arithmetical operation, a different protocol is used so that types for which only a subset of these operations makes sense can still work with those. Basic Usage Usually, Numbers is used in another module by using alias Numbers, as: N, followed by calling the functions using this aliased but still descriptive syntax: total_price = N.mult(N.add(price, fee), vat) Because Numbers dispatches based on protocol definitions, you only need to swap what kind of arguments are used to change the type of the result. Overloaded Operators As explicit opt-in functionality, Numbers will add overloaded operators to your module, if you write use Numbers, overload_operators: true This will alter a + b, a - b, a * b, a / b, -a and abs(a) to dispatch to the corresponding functions in the Numbers module. Do note that these overloaded operator versions are not allowed in guard tests, which is why this functionality is only provided as an opt-in option to use when an algorithm would be too unreadable without it. Examples: Using built-in numbers: iex&gt; alias Numbers, as: N iex&gt; N.add(1, 2) 3 iex&gt; N.mult(3,5) 15 iex&gt; N.mult(1.5, 100) 150.0 Using Decimals: (requires the Decimal library.) iex&gt; alias Numbers, as: N iex&gt; d = Decimal.new(2) iex&gt; N.div(d, 10) #Decimal&lt;0.2&gt; iex&gt; small_number = N.div(d, 1234) #Decimal&lt;0.001620745542949756888168557536&gt; iex&gt; N.pow(small_number, 100) #Decimal&lt;9.364478495445313580679473524E-280&gt; Defining your own Numbers implementations See Numbers.Protocols for a full explanation on how to do this.","ref":"Numbers.html","title":"Numbers","type":"module"},{"doc":"The absolute value of a number. Depends on an implementation existing of Numbers.Protocols.Absolute","ref":"Numbers.html#abs/1","title":"Numbers.abs/1","type":"function"},{"doc":"Adds two Numeric a and b together. Depends on an implementation existing of Numbers.Protocol.Addition","ref":"Numbers.html#add/2","title":"Numbers.add/2","type":"function"},{"doc":"","ref":"Numbers.html#add_id/1","title":"Numbers.add_id/1","type":"function"},{"doc":"Divides the Numeric a by b. Note that this is a supposed to be a full (non-truncated) division; no rounding or truncation is supposed to happen, even when calculating with integers. Depends on an implementation existing of Numbers.Protocol.Division","ref":"Numbers.html#div/2","title":"Numbers.div/2","type":"function"},{"doc":"Unary minus. Returns the negation of the number. Depends on an implementation existing of Numbers.Protocols.Minus","ref":"Numbers.html#minus/1","title":"Numbers.minus/1","type":"function"},{"doc":"Multiplies the Numeric a with the Numeric b Depends on an implementation existing of Numbers.Protocol.Multiplication","ref":"Numbers.html#mult/2","title":"Numbers.mult/2","type":"function"},{"doc":"","ref":"Numbers.html#mult_id/1","title":"Numbers.mult_id/1","type":"function"},{"doc":"Power function: computes base^exponent, where base is Numeric, and exponent has to be an integer. (This means that it is impossible to calculate roots by using this function.) Depends on an implementation existing of Numbers.Protocol.Exponentiation","ref":"Numbers.html#pow/2","title":"Numbers.pow/2","type":"function"},{"doc":"Subtracts the Numeric b from the Numeric a. Depends on an implementation existing of Numbers.Protocol.Subtraction","ref":"Numbers.html#sub/2","title":"Numbers.sub/2","type":"function"},{"doc":"Convert the custom Numeric struct to the built-in float datatype. This operation might be lossy, losing precision in the process.","ref":"Numbers.html#to_float/1","title":"Numbers.to_float/1","type":"function"},{"doc":"","ref":"Numbers.html#t:t/0","title":"Numbers.t/0","type":"type"},{"doc":"Helper functions that might make the implementation of Numbers for your own numberlike types easier.","ref":"Numbers.Helper.html","title":"Numbers.Helper","type":"module"},{"doc":"Performs &#39;Exponentiation by Squaring&#39;, which is a reasonably fast algorithm to compute integer powers, by performing log(n) multiplication steps. Depends on an implementation existing of Numbers.Protocols.Multiplication, as well as (to support negative powers) an implementation of Numbers.Protocols.Division.","ref":"Numbers.Helper.html#pow_by_sq/2","title":"Numbers.Helper.pow_by_sq/2","type":"function"},{"doc":"A set of protocols that can be implemented for your data structure, to add Numbers-support to it. In older versions of Numbers, structures were required to follow a single, very strict, behaviour. But because there are many different kind of structures that benefit from a numeric interface, including those for which one or multiple of these operations cannot be (unambiguously) defined, this has been split in these different protocols. By using the different protocols, each data structure can &#39;pick and choose&#39; what functionality is supported. As protocol dispatching is used, the result should be a lot faster than in older versions of Numbers, which performed behaviour-based runtime dispatch on the struct name. Coercion Numbers does not automatically transform numbers from one type to another if one of the functions is called with two different types. Frequently you do want to use other data types together with your custom data type. For this, a custom coercion can be specified, using Coerce.defcoercion/3 as exposed by the Coerce library that Numbers depends on.","ref":"Numbers.Protocols.html","title":"Numbers.Protocols","type":"module"},{"doc":"For supporting Numbers.abs/1.","ref":"Numbers.Protocols.Absolute.html","title":"Numbers.Protocols.Absolute","type":"protocol"},{"doc":"The absolute value of a number.","ref":"Numbers.Protocols.Absolute.html#abs/1","title":"Numbers.Protocols.Absolute.abs/1","type":"function"},{"doc":"","ref":"Numbers.Protocols.Absolute.html#t:t/0","title":"Numbers.Protocols.Absolute.t/0","type":"type"},{"doc":"For supporting Numbers.add/2.","ref":"Numbers.Protocols.Addition.html","title":"Numbers.Protocols.Addition","type":"protocol"},{"doc":"Adds two numbers together.","ref":"Numbers.Protocols.Addition.html#add/2","title":"Numbers.Protocols.Addition.add/2","type":"function"},{"doc":"Should return the &#39;additive identity&#39; of the same type as the argument. This is the value that can be added to another number, to let the result remain equal to that number. (For integers, this is 0, for floats, 0.0. Most other numeric types have their own &#39;zero&#39; variant as well.) This should be defined so that: a = some_num add(a, add_id()) == a add(add_id(), a) == a If the numeric structure also implements Numbers.Protocols.Subtraction, the following should also be true: a = some_num sub(a, add_id()) == a sub(add_id(), a) == a (Note that it is fine if the result is not structurally identical, as long as it is logically equal.)","ref":"Numbers.Protocols.Addition.html#add_id/1","title":"Numbers.Protocols.Addition.add_id/1","type":"function"},{"doc":"","ref":"Numbers.Protocols.Addition.html#t:t/0","title":"Numbers.Protocols.Addition.t/0","type":"type"},{"doc":"For supporting Numbers.div/2.","ref":"Numbers.Protocols.Division.html","title":"Numbers.Protocols.Division","type":"protocol"},{"doc":"Divides the rhs by the lhs. To be clear, this division operation is supposed to be precise.","ref":"Numbers.Protocols.Division.html#div/2","title":"Numbers.Protocols.Division.div/2","type":"function"},{"doc":"","ref":"Numbers.Protocols.Division.html#t:t/0","title":"Numbers.Protocols.Division.t/0","type":"type"},{"doc":"For supporting Numbers.pow/2.","ref":"Numbers.Protocols.Exponentiation.html","title":"Numbers.Protocols.Exponentiation","type":"protocol"},{"doc":"Power function, x^n. Unless a dedicated fast power algorithm exists for your data structure, you could use the &#39;Exponentiation by Squaring&#39; algorithm, by calling Numbers.Helper.pow_by_sq(num, integer_power) in the implementation, which is a reasonably fast algorithm that uses log(n) multiplication steps.","ref":"Numbers.Protocols.Exponentiation.html#pow/2","title":"Numbers.Protocols.Exponentiation.pow/2","type":"function"},{"doc":"","ref":"Numbers.Protocols.Exponentiation.html#t:t/0","title":"Numbers.Protocols.Exponentiation.t/0","type":"type"},{"doc":"For supporting Numbers.minus/1.","ref":"Numbers.Protocols.Minus.html","title":"Numbers.Protocols.Minus","type":"protocol"},{"doc":"Unary minus. Should return the negation of the number.","ref":"Numbers.Protocols.Minus.html#minus/1","title":"Numbers.Protocols.Minus.minus/1","type":"function"},{"doc":"","ref":"Numbers.Protocols.Minus.html#t:t/0","title":"Numbers.Protocols.Minus.t/0","type":"type"},{"doc":"For supporting Numbers.mult/2.","ref":"Numbers.Protocols.Multiplication.html","title":"Numbers.Protocols.Multiplication","type":"protocol"},{"doc":"Multiplies the two numbers together.","ref":"Numbers.Protocols.Multiplication.html#mult/2","title":"Numbers.Protocols.Multiplication.mult/2","type":"function"},{"doc":"Should return the &#39;multiplicative identity&#39; of the same type as the argument. This is the value that can be added to another number, to let the result remain equal to that number. (For integers, this is 1, for floats, 1.0. Most other numeric types have their own &#39;one&#39; variant as well.) This should be defined so that: a = some_num mult(a, mult_id()) == a mult(mult_id(), a) == a If the numeric structure also implements Numbers.Protocols.Division, the following should also be true: a = some_num div(a, mult_id()) == a div(mult_id(), a) == a (Note that it is fine if the result is not structurally identical, as long as it is logically equal.)","ref":"Numbers.Protocols.Multiplication.html#mult_id/1","title":"Numbers.Protocols.Multiplication.mult_id/1","type":"function"},{"doc":"","ref":"Numbers.Protocols.Multiplication.html#t:t/0","title":"Numbers.Protocols.Multiplication.t/0","type":"type"},{"doc":"For supporting Numbers.sub/2.","ref":"Numbers.Protocols.Subtraction.html","title":"Numbers.Protocols.Subtraction","type":"protocol"},{"doc":"Subtracts the rhs number from the lhs number.","ref":"Numbers.Protocols.Subtraction.html#sub/2","title":"Numbers.Protocols.Subtraction.sub/2","type":"function"},{"doc":"","ref":"Numbers.Protocols.Subtraction.html#t:t/0","title":"Numbers.Protocols.Subtraction.t/0","type":"type"},{"doc":"For supporting Numbers.to_float/1.","ref":"Numbers.Protocols.ToFloat.html","title":"Numbers.Protocols.ToFloat","type":"protocol"},{"doc":"Convert the custom Numeric struct to the built-in float datatype. It is okay to lose precision during this conversion.","ref":"Numbers.Protocols.ToFloat.html#to_float/1","title":"Numbers.Protocols.ToFloat.to_float/1","type":"function"},{"doc":"","ref":"Numbers.Protocols.ToFloat.html#t:t/0","title":"Numbers.Protocols.ToFloat.t/0","type":"type"}]